Module: ElasticGraph

Defined in:
lib/elastic_graph/query_registry/client_data.rb,
lib/elastic_graph/query_registry/config.rb,
lib/elastic_graph/query_registry/registry.rb,
lib/elastic_graph/query_registry/rake_tasks.rb,
lib/elastic_graph/query_registry/query_validator.rb,
lib/elastic_graph/query_registry/variable_dumper.rb,
lib/elastic_graph/query_registry/graphql_extension.rb,
lib/elastic_graph/query_registry/registration_status.rb,
lib/elastic_graph/query_registry/query_validators/for_registered_client.rb,
lib/elastic_graph/query_registry/query_validators/for_unregistered_client.rb,
lib/elastic_graph/query_registry/variable_backward_incompatibility_detector.rb

Overview

Copyright 2024 - 2026 Block, Inc.

Use of this source code is governed by an MIT-style license that can be found in the LICENSE file or at opensource.org/licenses/MIT.

frozen_string_literal: true

Defined Under Namespace

Modules: QueryRegistry