Class: ElasticGraph::QueryRegistry::VariableBackwardIncompatibilityDetector
- Inherits:
-
Object
- Object
- ElasticGraph::QueryRegistry::VariableBackwardIncompatibilityDetector
- Defined in:
- lib/elastic_graph/query_registry/variable_backward_incompatibility_detector.rb
Overview
Responsible for comparing old and new variable type info to see if any changes are backwards incompatible (and thus my break the client). Incompatibilities are identified by path and described.
Defined Under Namespace
Classes: Incompatibility
Instance Method Summary collapse
-
#detect(old_op_vars:, new_op_vars:) ⇒ Object
Entry point.
Instance Method Details
#detect(old_op_vars:, new_op_vars:) ⇒ Object
Entry point. Given the old variables for an operation, and the new variables for it, describes any backward incompatibilities in them.
16 17 18 |
# File 'lib/elastic_graph/query_registry/variable_backward_incompatibility_detector.rb', line 16 def detect(old_op_vars:, new_op_vars:) detect_incompatibilities(old_op_vars, new_op_vars, "$", "variable") end |